<?php

class EvClassRegistry
{
    /**
     * function to mimic ClassRegistry::init(), but works with our overriding model workflow
     * Pass PluginName.Model (normal models will work as normal) and it will check PluginNameModel first
     * then standard PluginName.Model
     *
     * @param   String  Model to load
     * @return  Object  Model object if found
     */
    public static function init($class, $strict = false)
    {
        $override = self::findOverrideModel($class);

        if ($override !== false && is_object($override)) {

            return $override;
        }

        // obviously none was found, try looking in the plugin
        return ClassRegistry::init($class, $strict);
    }


    /**
     * check for overriding model
     *
     * @param   string          model to try and load
     * @return  object|bool     Loaded model or false on fail
     */
    public static function findOverrideModel($model)
    {
        if (is_array($model) && empty($model['class'])) {
            return false;
        }

        $modelClass = $model;
        if (is_array($model) && ! empty($model['class'])) {
            $modelClass = $model['class'];
        }

        if (strpos($modelClass, '.') !== false) {

            list($pluginName, $modelName) = pluginSplit($modelClass);

            if (is_array($model)) {
                $model['class'] = $pluginName . $modelName;
            } else {
                $model = $pluginName . $modelName;
            }

            // first check if we can find an 'overriding model'
            // true parameter will make it return false if none was found

            $Model = ClassRegistry::init($model, true);
            if (
                $Model !== false &&
                is_object($Model) &&
                $Model instanceof $pluginName . $modelName
            ) {

                // return the model
                return $Model;
            }
        }

        return false;
    }


    /**
     * given the model object, try and work out the plugin.name version of the model
     * incase any models have been extend, init method above can be used to load the model
     *
     * @param Object    Model Object
     * @return string   Model name in the form of Plugin.Model (or just Model if none plugin)
     */
    public static function getName($Model)
    {
        if (! empty($Model->plugin)) {

            return $Model->plugin . '.' . $Model->name;

        } else {

            $className = get_class($Model);
            $pluginName = str_replace($Model->name, '', $className);

            if (! empty($pluginName) && CakePlugin::loaded($pluginName)) {

                return $pluginName . '.' . $Model->name;
            }
        }

        return $Model->name;
    }
}
